import { all, fork, call, put, takeEvery, select } from 'redux-saga/effects';
import errorHandler from 'stores/sagas/commonSaga';
import { SAGA_ACTIONS } from 'common/constants';
import { formatCreatedZone, formatEditedZone, isGeometryChanged } from 'utils/zoneUtils';
import { closeZoneEditAdminPageModal, closeZoneEditSitePageModal } from 'stores/actions/modalActions/zoneModalActions';
import { groupsMap, trackablesMap, zoneEditSiteViewVertices, zonePrototypeList } from 'stores/sagas/selectors';
import { createZone, deleteSingleZone, editZone } from 'stores/sagas/zone/zoneSagas';
import { siteViewClearMode } from 'stores/actions/siteViewActions';


function* editZoneModal(zone) {
    try {
        const trackables = yield select(trackablesMap);
        const groups = yield select(groupsMap);
        const zoneUid = zone.uid;
        const zonesPrototypes = yield select(zonePrototypeList);
        const editedGeometry = yield select(zoneEditSiteViewVertices);
        const geometryChanged = yield call(isGeometryChanged, editedGeometry, zone?.zone_geometry?.positions);

        if (editedGeometry.length > 0 && geometryChanged) {
            yield call(deleteSingleZone, zoneUid);
            const newZone = yield call(formatCreatedZone, zonesPrototypes, { ...zone }, { zone_geometry: { positions: editedGeometry } }, trackables, groups);
            yield call(createZone, newZone);
        } else {
            const updatedZone = yield call(formatEditedZone, zone, trackables, groups, zonesPrototypes);
            yield call(editZone, zoneUid, updatedZone);
        }
        yield put(siteViewClearMode());
    } catch (err) {
        yield call(errorHandler, err);
    }
}

function* editSitePageZone(action) {
    try {
        yield call(editZoneModal, action.params);
        yield put(closeZoneEditSitePageModal());
    } catch (err) {
        yield call(errorHandler, err);
    }
}

function* editAdminPageZone(action) {
    try {
        yield call(editZoneModal, action.params);
        yield put(closeZoneEditAdminPageModal());
    } catch (err) {
        yield call(errorHandler, err);
    }
}


function* watchEditSitePageZone() {
    yield takeEvery(SAGA_ACTIONS.SAGA_ZONES_EDIT_SITE_PAGE, editSitePageZone);
}

function* watchEditAdminPageZone() {
    yield takeEvery(SAGA_ACTIONS.SAGA_ZONES_EDIT_ADMIN_PAGE, editAdminPageZone);
}

export default function* zoneEditSaga() {
    return yield all([
        fork(watchEditSitePageZone),
        fork(watchEditAdminPageZone),
    ]);
}
